from distutils.core import setup
setup(
  name = 'clustergrammer',
  packages = ['clustergrammer'], # this must be the same as the name above
  version = '1.7.1',
  description = 'A python module for the Clustergrammer visualization project',
  author = 'Nicolas Fernandez',
  author_email = 'nickfloresfernandez@gmail.com',
  url = 'https://github.com/MaayanLab/clustergrammer-py',
  download_url = 'https://github.com/MaayanLab/clustergrammer-py/tarball/1.1.2',
  keywords = ['testing'],
  classifiers = [],
)